;*********************************************************************
;Interrupt handler. Scelta dell'interrupt generato
        ORG     0x04
        movwf   W_Temp          ;Salvataggio registro w (da qualsiasi banco)
        swapf   STATUS,w        ;Salvataggio registro STATUS (da qualsiasi banco)
        movwf   St_Temp

        BANK0                   ;Individua banco 0 (nel main  richiamato
                                ;BANK1 durante lettura EEprom
        btfsc   INTCON,INTF     ;Verifica se generato l'Interrupt su RB0
        goto    rb0_int         ;Salta a Interrupt rb0 se INTF=1
        goto    tim_int         ;Altrimenti salta a Timer Interrupt

end_ih  swapf   St_Temp,w       ;Riprisino registro STATUS (da qualsiasi banco)
        movwf   STATUS
        swapf   W_Temp,f        ;Ripristino registro w (da qualsiasi banco)
        swapf   W_Temp,w
        retfie

;-----------------------------------------------------
;*** Interrupt Timer ***
;Si genera questo interrupt quando il registro TMR0, continuamente incrementato
;(ogni 2 us), arriva a 0. E'utilizzato per decrementare da 255 a 0 la
;variabile CntPwm che stabilisce i 'passi' del ritardo di conduzione del
;triac rispetto al passaggio sullo zero della tensione sinosoidale (Pwm).

;Inoltre pone a 1 logico le 10 uscite collegate ai triac quando 
;VALORE_USCITA>=CntPwm.
;Il triac rimarr in conduzione fino al prossimo passaggio sullo zero.

;Poich, con bassa luminosit, una modifica del suo valore appare a vista
;in un incremento o decremento non proprio graduale (a gradini),
;si preferisce rendere lo scatto della parzializzazione pi fitto
;durante la parte discendente della tensione sinosoidale.
;Quindi si esegue tim_int utilizzando due tempi differenti
; - CntPwm (255...161) --> luminosit elevata, scatto parzializz. maggiore 54us
; - CntPwm (160...0)   --> luminosit minima, scatto parzializz. minore 25us

tim_int                         ;Ricarica TMR0 per eseguire il prossimo
        movf    CntPwm,w        ;tim_int secondo il valore di CntPwm
        sublw   D'160'          ;w=160-CntPwm (setta flag carry se CntPwm<=160)
        movlw   D'246'          ;ricarica TMR0 per esecuzione prossimo tim_int
        btfss   STATUS,C        ;(256-246) * 2us = 20us + 5us(scelta) ==> 25us
        movlw   D'237'          ;(256-237) * 2us = 38us + 5us(scelta) ==> 43us
        movwf   TMR0            ;25us * 160 volte = 4000us
                                ;43us * 95 volte =  4085us
                                ;Totale    --->     8085us  (circa 8.3ms)

        btfsc   I_100HZ         ;Esce se il segnale di zero-crossing  alto
        goto    tim_e

        movf    CntPwm,w        ;Uscita 1
        subwf   Light+0x00,w    ;w = Light+0x00 - CntPwm
        btfsc   STATUS,C        ;Se Light+0x00<CntPwm salta prossima istruzione
        bsf     O_OUT1          ;Porta in conduzione il triac che pilota la
                                ;uscita 1 in ritardo rispetto allo 0

        movf    CntPwm,w        ;Uscita 2
        subwf   Light+0x01,w
        btfsc   STATUS,C
        bsf     O_OUT2

        movf    CntPwm,w        ;Uscita 3
        subwf   Light+0x02,w
        btfsc   STATUS,C
        bsf     O_OUT3

        movf    CntPwm,w        ;Uscita 4
        subwf   Light+0x03,w
        btfsc   STATUS,C
        bsf     O_OUT4

        movf    CntPwm,w        ;Uscita 5
        subwf   Light+0x04,w
        btfsc   STATUS,C
        bsf     O_OUT5

        movf    CntPwm,w        ;Uscita 6
        subwf   Light+0x05,w
        btfsc   STATUS,C
        bsf     O_OUT6

        movf    CntPwm,w        ;Uscita 7
        subwf   Light+0x06,w
        btfsc   STATUS,C
        bsf     O_OUT7

        movf    CntPwm,w        ;Uscita 8
        subwf   Light+0x07,w
        btfsc   STATUS,C
        bsf     O_OUT8

        movf    CntPwm,w        ;Uscita 9
        subwf   Light+0x08,w
        btfsc   STATUS,C
        bsf     O_OUT9

        movf    CntPwm,w        ;Uscita 10
        subwf   Light+0x09,w
        btfsc   STATUS,C
        bsf     O_OUT10

        movf    CntPwm,f        ;Se CntPwm<>0 decrementa CntPwm
        btfss   STATUS,Z        ;Salta prossima se CntPwm=0
        decf    CntPwm,f        ;Decrementa CntPwm

tim_e   bcf     INTCON,T0IF     ;Riabilita interrupt su TMR0
        goto    end_ih


;-----------------------------------------------------
;*** Interrupt RB0 ***
;Si genera questo interrupt quando l'ingresso RB0 passa da 0 a 1 in presenza
;dello 'zero' della tensione sinosoidale.
;E' utilizzato per porre a 0 le 10 uscite e ad inizializzare la variabile CntPwm.
;Inoltre:
; - Incrementa i contatori C10msH,L ogni 10ms (100Hz) fino al raggiungimento
;   del valore T10msH,L che definisce il tempo del singolo passo.
; - Azzera il contatore per non entrare nella routine di comunicazione.
;   Quando l'utente sposta l'apposito deviatore nella modalit comunicazione,
;   questa routine non  pi eseguita e il mainloop entra nella modalit
;   comunicazione. E' necessario un reset per ritornare al funzionamento normale

rb0_int bcf     O_OUT1          ;Pone a 0 le 10 uscite
        bcf     O_OUT2
        bcf     O_OUT3
        bcf     O_OUT4
        bcf     O_OUT5
        bcf     O_OUT6
        bcf     O_OUT7
        bcf     O_OUT8
        bcf     O_OUT9
        bcf     O_OUT10

        movlw   D'255'          ;Inizializza CntPwm a 255
        movwf   CntPwm

        btfsc   F_TSTEP         ;Salta se il tempo  gi trascorso
        goto    rb0_2

        incfsz  C10msL,f        ;Incrementa contatori C10msH,L
        goto    rb0_1
        incf    C10msH,f
        call    toggle_led_time ;F lampeggiare il led Time ogni 2.5 secondi circa 
                                ;per tempi maggiori di 20 minuti (opzione)

rb0_1   movf    C10msH,w        ;Se C10msH,L = T10msH,L pone Flag a 1
        xorwf   T10msH,w
        btfss   STATUS,Z
        goto    rb0_2
        movf    C10msL,w
        xorwf   T10msL,w
        btfss   STATUS,Z
        goto    rb0_2
        bsf     F_TSTEP

rb0_2
;       movlw   D'190'		;Attesa 380us per non far eseguire tim_int
;	movwf   CntZero		;quando la tensione sinosoidale non  ancora
;rb0_3   nop                     ;a zero.
;        decfsz  CntZero,f
;        goto    rb0_3

        clrf    CComm           ;Azzera contatore CComm per non far andare
                                ;il mainloop nella routine di comunicazione
        bcf     INTCON,INTF     ;Riabilita interrupt su RB0
        goto    end_ih
