;*********************************************************************
;Interrupt handler. Scelta dell'interrupt generato
        ORG     0x04
        movwf   W_Temp          ;Salvataggio registro w (da qualsiasi banco)
        swapf   STATUS,w        ;Salvataggio registro STATUS (da qualsiasi banco)
        movwf   St_Temp
        BANK0                   ;Individua banco 0 (nel main  richiamato
                                ;BANK1 durante lettura EEprom

;*** Interrupt Timer0 ***
;Si genera questo interrupt quando il registro TMR0, continuamente incrementato
;(ogni 2 us), arriva a 0.

;Decrementa da 255 a 0 la variabile CntPwm che stabilisce i 'passi' del Pwm
;da 0% al 100% per variare la luminosit della singola uscita:
; -Fronte positivo: setta ciascuna delle 10 uscite quando Valore_Uscita>=CntPwm.
; -Fronte negativo: resetta tutte e 10 le uscite quando CntPwm=0

;Poich, con bassa luminosit, una modifica del Pwm appare a vista
;in un incremento o decremento non proprio graduale (a gradini),
;si preferisce rendere lo scatto del Pwm pi fitto quando  minore del 50%
;Quindi si esegue tim_int utilizzando due tempi differenti
; - CntPwm (255...142) --> luminosit elevata, passo Pwm maggiore (54us)
; - CntPwm (141...0)   --> luminosit minima, passo Pwm minore (25us)

                                ;Ricarica TMR0 per eseguire il prossimo
        movf    CntPwm,w        ;timer_interrupt secondo il valore di CntPwm
        sublw   D'141'          ;w=141-CntPwm (setta flag carry se CntPwm<=141)
        movlw   D'244'          ;ricarica TMR0 per esecuzione prossimo tim_int
        btfss   STATUS,C        ;(256-244) * 2us = 22us + 2.5us(scelta) ==> 26.5us
        movlw   D'230'          ;(256-230) * 2us = 52us + 2.5us(scelta) ==> 54.5us
        movwf   TMR0            ;26.5us * 141 volte =  3736.5 us
                                ;54.5us * 115 volte =  6267.5 us
                                ;Totale    --->       10004.0 us  (freq.Pwm = 99.96Hz)

;        movlw   D'238'         ;Per Pwm con 256 passi uguali di 39us
;        movwf   TMR0

        movf    CntPwm,f        ;Se contatore per duty-cycle =0
        btfss   STATUS,Z
        goto    tim_2
        bcf     O_OUT1          ;Pone a 0 le 10 uscite per
        bcf     O_OUT2          ;generare il fronte negativo del Pwm
        bcf     O_OUT3
        bcf     O_OUT4
        bcf     O_OUT5
        bcf     O_OUT6
        bcf     O_OUT7
        bcf     O_OUT8
        bcf     O_OUT9
        bcf     O_OUT10

        movlw   D'255'          ;Inizializza CntPwm a 255
        movwf   CntPwm

        btfsc   F_TSTEP         ;Salta se il tempo  gi trascorso
        goto    tim_2

        incfsz  C10msL,f        ;Incrementa contatori C10msH,L
        goto    tim_1
        incf    C10msH,f
        call    toggle_led_time ;F lampeggiare il led Time ogni 2.5 secondi circa
                                ;per tempi maggiori di 20 minuti (opzione)

tim_1   movf    C10msH,w        ;Se C10msH,L = T10msH,L pone Flag a 1
        xorwf   T10msH,w
        btfss   STATUS,Z
        goto    tim_2
        movf    C10msL,w
        xorwf   T10msL,w
        btfss   STATUS,Z
        goto    tim_2
        bsf     F_TSTEP

tim_2   movf    CntPwm,w        ;Uscita 1
        subwf   Light+0x00,w    ;w = Light+0x00 - CntPwm
        btfsc   STATUS,C        ;Se Light+0x00<CntPwm salta prossima istruzione
        bsf     O_OUT1          ;Genera il fronte positivo del Pwm relativo a Uscita1

        movf    CntPwm,w        ;Uscita 2
        subwf   Light+0x01,w
        btfsc   STATUS,C
        bsf     O_OUT2

        movf    CntPwm,w        ;Uscita 3
        subwf   Light+0x02,w
        btfsc   STATUS,C
        bsf     O_OUT3

        movf    CntPwm,w        ;Uscita 4
        subwf   Light+0x03,w
        btfsc   STATUS,C
        bsf     O_OUT4

        movf    CntPwm,w        ;Uscita 5
        subwf   Light+0x04,w
        btfsc   STATUS,C
        bsf     O_OUT5

        movf    CntPwm,w        ;Uscita 6
        subwf   Light+0x05,w
        btfsc   STATUS,C
        bsf     O_OUT6

        movf    CntPwm,w        ;Uscita 7
        subwf   Light+0x06,w
        btfsc   STATUS,C
        bsf     O_OUT7

        movf    CntPwm,w        ;Uscita 8
        subwf   Light+0x07,w
        btfsc   STATUS,C
        bsf     O_OUT8

        movf    CntPwm,w        ;Uscita 9
        subwf   Light+0x08,w
        btfsc   STATUS,C
        bsf     O_OUT9

        movf    CntPwm,w        ;Uscita 10
        subwf   Light+0x09,w
        btfsc   STATUS,C
        bsf     O_OUT10

        decf    CntPwm,f        ;Decrementa CntPwm

tim_e   bcf     INTCON,T0IF     ;Riabilita interrupt su TMR0

        swapf   St_Temp,w       ;Riprisino registro STATUS (da qualsiasi banco)
        movwf   STATUS
        swapf   W_Temp,f        ;Ripristino registro w (da qualsiasi banco)
        swapf   W_Temp,w
        retfie

