;************************************************************************
; PRESEPE6_AC50.ASM Rev.1
; Versione Corrente Alternata 50Hz
; 20/4/2013 by Sasa

; Generatore programmabile di effetti luminosi e uscite on-off
; Le variazioni di luminosit, le accensioni e i tempi del ciclo
; (suddivisi in 8 fasi temporali) possono essere programmate a piacimento
; attraverso il software che dialoga sulla porta seriale
; Maggiori informazioni nel file PRESEPE6.PDF
;************************************************************************

        PROCESSOR       16F628                  ;Tipo di processore
        INCLUDE         "P16F628.INC"           ;inclusione File .inc
        __CONFIG        3D6EH                   ;Nessuna protezione codice
                                                ;Disabilitato LVP
                                                ;Brown-out abilitato
                                                ;Funzione MCLR attiva
                                                ;Power Up abilitato
                                                ;Watch-dog abilitato
                                                ;Oscillatore tipo HS (16 MHz)

;*********************************************************************
;Definizione Macro per selezionare i 3 banchi di memoria Ram
BANK0	MACRO
	bcf     STATUS,RP1
	bcf     STATUS,RP0
        ENDM
BANK1	MACRO
	bcf     STATUS,RP1
	bsf     STATUS,RP0
	ENDM
BANK2	MACRO
	bsf     STATUS,RP1
	bcf     STATUS,RP0
	ENDM

;************************************************************************
;Dichiarazione costanti I/O
#DEFINE O_OUT1   PORTB,7        ;Uscita 1  (alba)
#DEFINE O_OUT2   PORTB,6        ;Uscita 2  (giorno)
#DEFINE O_OUT3   PORTB,5        ;Uscita 3  (tramonto)
#DEFINE O_OUT4   PORTB,4        ;Uscita 4  (notte)
#DEFINE O_OUT5   PORTB,3        ;Uscita 5  (stelle)
#DEFINE O_OUT6   PORTB,2        ;Uscita 6  (zona 1)
#DEFINE O_OUT7   PORTB,1        ;Uscita 7  (zona 2)
#DEFINE O_OUT8   PORTA,4        ;Uscita 8  (zona 3)
#DEFINE O_OUT9   PORTA,3        ;Uscita 9  (zona 4)
#DEFINE O_OUT10  PORTA,2        ;Uscita 10 (zona 5)

#DEFINE I_PMENU  PORTA,1        ;Ingresso pulsante MENU
#DEFINE O_LTIME  PORTA,0        ;Uscita led TIME
#DEFINE I_100HZ  PORTB,0        ;Ingresso sincronismo di rete 100Hz

#DEFINE I_RX     PORTB,0        ;Ingresso seriale dati (duplicato)
#DEFINE O_TX     PORTA,0        ;Uscita seriale dati   (duplicato)


;************************************************************************
;Dichiarazione flag all'interno della variabile Flags
#DEFINE F_LTINIT   Flags,7      ;Inizializzazione effetti dissolvenza
                                ;ad inizio ciclo
#DEFINE F_LDEFFP2  Flags,6      ;Caricamento degli effetti Parte2 di
                                ;ogni fase temporale
#DEFINE F_TSTEP    Flags,5      ;Raggiunto tempo per singolo passo


;************************************************************************
;Dichiarazione costanti
MAXEEA  EQU     D'128'          ;Locazioni EEprom utilizzate per
                                ;impostazioni utente (0..127)

RND_EN  EQU     0x03            ;Bit Stato 'on-off Random' uscita in var LtEffM(x)
ENABLE  EQU     0x03            ;Bit Stato Abilitazione uscita in var LtEffM(x)
                                ;(lo stesso bit che assume 2 diverse funzioni)
FLICKER EQU     0x01            ;Bit Stato Tremolo uscita in var LtEffM(x)


;************************************************************************
;Dichiarazione variabili Ram
        ORG     0x20
Light   RES     0x0A            ;Luminosit uscite 1..10 (0=off..255=max)
                                ;per comandare in ritardo i triac

LtValue RES     0x0A            ;Valore assoluto luminosit uscite (0=off..255=max)
                                ;(senza modifiche dovute all'effetto tremolo)

LtEffM  RES     0x0A            ;Andamento + stati uscite 1..10
                                ;bit 7,6,5,4 = Effetti (16 andamenti)
                                ;bit 3 = Stato random / Abilitazione (on-off)
                                ;bit 2,1,0 = Tipo Tremolo (8 modalit)

CStepH  RES     0x01            ;Contatore passi del ciclo giornata
CStepL  RES     0x01            ;(512 passi x 8 fasi = 4096 passi max)

T10msH  RES     0x01            ;Tempo del singolo passo della fase temporale
T10msL  RES     0x01            ;impostato da utente (moltiplicare *10ms)

C10msH  RES     0x01            ;Contatore incrementato ogni 10ms sul fronte
C10msL  RES     0x01            ;positivo di RB0 (Frequenza rete 50x2=100Hz)

CntPwm  RES     0x01            ;Contatore per ritardo conduzione triac
;CntZero RES     0x01	        ;Ritardo per attesa zero-crossing reale

RandomH RES     0x01            ;Generatore numeri casuali per tremolo e random on-off
RandomL RES     0x01
DelRnd  RES     0x01            ;Tempo (10..100ms) ritardo variabile per numeri casuali
CntRnd  RES     0x01            ;Contatore per ritardo variabile numeri casuali

SpdFixH RES     0x01            ;Contatori per generazione velocit fisse
SpdFixL RES     0x01

Menu    RES     0x01            ;Accesso ai menu utente 1..9

Value   RES     0x01            ;Appoggio temporaneo valore per vari usi
Temp    RES     0x01            ;Appoggio temporaneo per vari usi
Flags   RES     0x01            ;Flag di stato per vari utilizzi (vedi #DEFINE)

StPMenu RES     0x01            ;Stato pulsante Menu
CPMenuH RES     0x01            ;Contatore per pressione/timeout pulsante Menu
CPMenuL RES     0x01

CComm   RES     0x01            ;Contatore per accedere alla routine comunicazione

RegTxRx RES     0x01            ;Carattere da trasmettere/ricevuto da seriale
CBit    RES     0x01            ;Contatore bit trasmessi/ricevuti
CDelBit RES     0x01            ;Contatore ritardo per singolo bit

Count1  RES     0x01            ;Contatori per vari utilizzi
Count2  RES     0x01

CntDel1 RES     0x01            ;Contatori per ritardi software
CntDel2 RES     0x01
CntDel3 RES     0x01

;Dichiarazione variabili Ram (accessibili da qualsiasi banco)
        ORG     0x70
W_Temp  RES     0x01            ;Salvataggio registro W durante Interrupt
St_Temp RES     0x01            ;Salvataggio registro STATUS durante Interrupt


;************************************************************************
;Inizio memoria programma (locazione 00)
        ORG     0x00
        goto    main_start


;************************************************************************
;Inclusione routine per gestioni varie
        INCLUDE "IRQS.INC"      ;Interrupt
        INCLUDE "SUBS.INC"      ;Subroutine varie
        INCLUDE "CYCLE.INC"     ;Gestione ciclo funzionamento normale
        INCLUDE "MENU.INC"      ;Gestione men utente
        INCLUDE "COMM.INC"      ;Comunicazione seriale


;************************************************************************
;Programma PRINCIPALE

;Configurazione porte I/O (vedi #DEFINE all'inizio) e settaggi vari (1=IN 0=OUT)
main_start   
        BANK0                   ;Ram Banco 0
        clrf    PORTA
        clrf    PORTB
        movlw   0x07            ;Abilita i pin RA0..RA3 come I/O standard
        movwf   CMCON
        BANK1                   ;Ram Banco 1
        movlw   B'11100010'     ;Configurazione Porta A
        movwf   TRISA
        movlw   B'00000001'     ;Configurazione Porta B
        movwf   TRISB
        movlw   B'11000010'     ;bit7: Disattiva circuiteria weak-up ingressi
        movwf   OPTION_REG      ;bit6: Attiva interrupt RB0 su fronte salita
                                ;bit5: Ingresso prescaler a fOsc/4 (4 MHz)
                                ;bit3: Ingresso registro TMR0 a Prescaler
                                ;bit2,1,0: Divisione prescaler 1:8
                                ;Frequenza uscita prescaler = 500000 Hz
        BANK0

        movlw   B'00110000'     ;Abilita Interrupt su Timer e su porta RB0
        movwf   INTCON          ;Manca l'abilitazione globale (bit 7)

        call    out_irq_off     ;Disattiva tutte le uscite

        call    del_1s          ;Se premuto pulsante dopo reset per t>2sec
        btfsc   I_PMENU         ;carica valori di default in eeprom  (menu 0)
        goto    main_1
        call    del_1s
        btfsc   I_PMENU
        goto    main_1
        call    menu0
        goto    main_start      ;Riavvia il programma


                                ;*** Inizializza variabili per ciclo normale (puntatore)
main_1  movlw   Light+0x00      ;Azzera tutte le variabili
        movwf   FSR
        movlw   D'52'
        movwf   Count1
main_2  clrf    INDF
        incf    FSR,f
        decfsz  Count1,f
        goto    main_2
        movlw   D'10'           ;Inizializza ritardo per generatore
        movwf   DelRnd          ;di numeri casuali al minimo
        movwf   CntRnd

        bsf     INTCON,GIE      ;Abilita gli interrupt precedentemente selezionati
        bsf     F_TSTEP         ;Forza il flag tempo per avviare la prima fase

main_loop                       ;*** Loop principale
        incf    CComm,f         ;Verifica se si vuole entrare nella gestione
        movf    CComm,w         ;comunicazione. CComm  continuamente azzerata
        xorlw   D'250'          ;dall'irq rb0 durante il funzionamento normale.
        btfsc   STATUS,Z        ;Raggiunge 250 quando irq rb0 non viene pi
        goto    make_comm       ;eseguito perch l'utente ha spostato l'apposito
                                ;deviatore per la comunicazione.

        call    check_pmenu     ;Verifica se il pulsante PMenu  stato premuto 2 volte
        movf    StPMenu,w
        xorlw   0x04
        btfss   STATUS,Z
        goto    run_cycle       ;Se non premuto, esegue il ciclo di funzionamento normale
        goto    choose_menu     ;Se premuto 2 volte entra nel men utente



;************************************************************************
;Inclusione Tabelle Rom
        INCLUDE "TABLES.INC"    ;Tabelle rom


        END                     ;Fine programma

