;-----------------------------------------------------
;Inverte lo stato del led Time per generare il lampeggio
; Input:  O_LTIME
; Output: O_LTIME

toggle_led_time
        btfsc   O_LTIME
        goto    blt_1
        bsf     O_LTIME         ;Accende il led
        return
blt_1   bcf     O_LTIME         ;Spegne il led
        return


;-----------------------------------------------------
;Verifica se il pulsante PMENU  stato premuto 2 volte entro un tempo di 2 secondi
;per entrare nel men utente
; Input:  I_PMENU, StPMenu
; Output: StPMenu
; Usate:  CPMenuH, CPMenuL

check_pmenu
        btfss   I_PMENU         ;Verifica se il pulsante  premuto
        goto    cp_3
        movf    StPMenu,w       ;Pulsante non premuto
        xorlw   0x01
        btfss   STATUS,Z        ;Se stato pulsante =1 lo pone =2
        goto    cp_1
        movlw   0x02
        goto    cp_5
cp_1    movf    StPMenu,w       ;Se stato pulsante =3 lo pone =4
        xorlw   0x03            ;per entrare nel men utente
        btfss   STATUS,Z
        goto    cp_2
        movlw   0x04
        movwf   StPMenu
        return
cp_2    movf    StPMenu,w       ;Se stato pulsante =2 salta per incremento timeout
        xorlw   0x02            ;altrimenti azzera variabili
        btfsc   STATUS,Z
        goto    cp_6
        clrf    StPMenu
        clrf    CPMenuH
        clrf    CPMenuL
        return

cp_3    movf    StPMenu,w       ;Pulsante premuto
        btfss   STATUS,Z        ;Se stato pulsante =0 e timeout scaduto, esce
        goto    cp_4
        btfsc   CPMenuH,3
        return
        movlw   0x01            ;Se stato pulsante =0 e timeout non scaduto, pone stato =2
        goto    cp_5
cp_4    xorlw   0x02            ;Se stato pulsante =0 lo pone =3
        btfss   STATUS,Z
        goto    cp_6
        movlw   0x03

cp_5    movwf   StPMenu         ;Aggiorna variabile di stato e attende 10ms
        call    deli_10ms
cp_6    btfss   CPMenuH,3       ;Verifica se il contatore timeout  al massimo (2048)
        goto    cp_7
        clrf    StPMenu
        return
cp_7    incf    CPMenuL,f       ;Incrementa il contatore timeout
        btfsc   STATUS,Z
        incf    CPMenuH,f
        return


;-----------------------------------------------------
;Disattiva interrupt e spegne tutte le uscite
; Output: O_OUT1 ... O_OUT10

out_irq_off
        bcf     INTCON,GIE      ;Disattiva irq

        bcf     O_OUT1          ;Pone a 0 le 10 uscite
        bcf     O_OUT2
        bcf     O_OUT3
        bcf     O_OUT4
        bcf     O_OUT5
        bcf     O_OUT6
        bcf     O_OUT7
        bcf     O_OUT8
        bcf     O_OUT9
        bcf     O_OUT10

        bcf     O_LTIME         ;Spegne il led Time
        return


;-----------------------------------------------------
;Legge dalla memoria EEprom il dato all'indirizzo specificato in  w
; Input:  w     Indirizzo di lettura dalla EEprom
; Output: w     Dato letto dalla EEprom

read_eeprom
        BANK1
        movwf   EEADR           ;Copia w (indirizzo) in EEADR
        bsf     EECON1,RD
        movf    EEDATA,w        ;Ritorna il dato letto in w
        BANK0
        return


;-----------------------------------------------------
;Scrive sulla memoria EEprom il dato all'indirizzo specificato
; Input:  EEADR         Indirizzo di scrittura
; Input:  EEDATA        Dato da scrivere
; Output: EEprom

write_eeprom
;        BANK1                  ;Gi in BANK1
        bcf     INTCON,GIE      ;Disabilita tutti gli interrupt
        bsf     EECON1,WREN     ;Sequenza obbligata
        movlw   0x55
        movwf   EECON2
        movlw   0xAA
        movwf   EECON2
        bsf     EECON1,WR       ;Start scrittura
wee_1   clrwdt                  ;Aspetta finch  completata la scrittura
        btfsc   EECON1,WR
        goto    wee_1
;        bcf     EECON1,WR       ;Inutile: viene resettato da hardware
        BANK0
        return


;-----------------------------------------------------
;Effettua ritardi da 100ms a 2 secondi (senza interrupt)
;Con gli interrupt abilitati durano circa 3 volte di pi

; Usate: CntDel1, CntDel2, CntDel3

del_5s                          ;Ritardo 5 secondi (50 * 100 ms)
        movlw   D'50'
        goto    dels_1

del_1s                          ;Ritardo 1 secondo (10 * 100 ms)
        movlw   D'10'
        goto    dels_1

del_500ms                       ;Ritardo 0,5 secondi (5 * 100 ms)
        movlw   D'5'
        goto    dels_1

del_100ms                       ;Ritardo 0,1 secondi (1 * 100 ms)
        movlw   D'1'
dels_1  movwf   CntDel3

dels_2  movlw   D'196'          ;Ritardo [(8*255)+3]*196=400428/4cm = 100107us
        movwf   CntDel2
        clrf    CntDel1
dels_3  clrwdt                  ;Ciclo 1
        nop
        nop
        nop
        nop
        decfsz  CntDel1,f
        goto    dels_3
        decfsz  CntDel2,f       ;Ciclo 2
        goto    dels_3

        decfsz  CntDel3,f       ;Ciclo 3 ripete ritardo 100ms
        goto    dels_2
        return


;-----------------------------------------------------
;Effettua ritardi da 1ms a 100ms (con interrupt abilitati)
;Con gli interrupt non abilitati durano circa 2 volte di meno

; Usate: CntDel1, CntDel2

deli_100ms                      ;Ritardo 100ms (100 * 1ms)
        movlw   D'100'
        goto    deli_1

deli_10ms                       ;Ritardo 10ms (10 * 1ms)
        movlw   D'10'
        goto    deli_1

deli_1ms                        ;Ritardo 1ms (1 * 1ms)
        movlw   D'1'
deli_1  movwf   CntDel2

deli_2  movlw   D'220'          ;Ritardo 1ms con irq on (tempo reale / 3 circa)
        movwf   CntDel1
deli_3  clrwdt                  ;[(10 * 220) / 4cm] * 2 = 1100 us --> 1ms circa
        nop
        nop
        nop
        nop
        nop
        nop
        decfsz  CntDel1,f       ;Ciclo 1
        goto    deli_3

        decfsz  CntDel2,f       ;Ciclo 2 ripete ritardo 1ms
        goto    deli_2
        return

